from PySide2 import QtWidgets


class DirectoryWidget(QtWidgets.QWidget):
    def __init__(self, parent=None, default_dir: str = "", title: str = "Destination", browse_title: str = "Select directory"):
        """
        Widget containing groupBox, lineEdit and button widgets.
        Button triggers QFileDialog.getExistingDirectory method and sets lineEdit text to path returned from it.

        :param parent: Widget parent, defaults to None
        :type parent: QtWidget, optional
        :param default_dir: Default text to use for lineEdit text, defaults to ""
        :type default_dir: str, optional
        :param title: Group box title, defaults to "Destination"
        :type title: str, optional
        :param browse_title: QFileDialog window title, defaults to "Select directory"
        :type browse_title: str, optional
        """
        super().__init__(parent)
        self.default_dir = default_dir
        self.title = title
        self.browse_title = browse_title

        self.create_widgets()
        self.create_layouts()
        self.create_connections()

    def create_widgets(self):
        self.group = QtWidgets.QGroupBox(self.title)
        self.line_edit = QtWidgets.QLineEdit(self.default_dir)
        self.browse_button = QtWidgets.QPushButton("Browse...")

    def create_layouts(self):
        self.main_layout = QtWidgets.QVBoxLayout()
        self.setLayout(self.main_layout)
        self.group_layout = QtWidgets.QHBoxLayout()
        self.group_layout.addWidget(self.line_edit)
        self.group_layout.addWidget(self.browse_button)
        self.group.setLayout(self.group_layout)
        self.main_layout.addWidget(self.group)
        self.main_layout.addStretch()

    def create_connections(self):
        self.browse_button.clicked.connect(self.browse_path)

    def browse_path(self):
        """Get directory path from QFileDialog"""
        path = QtWidgets.QFileDialog.getExistingDirectory(self, self.browse_title, self.line_edit.text())
        if path:
            self.line_edit.setText(path)



import os
import pathlib
import platform
import requests
import shutil
from zipfile import ZipFile
from PySide2 import QtCore
from PySide2 import QtWidgets
from github import Github
from github import Repository
from github import GithubException
from simple_installer import Logger


class Installer(QtCore.QObject):
    # Progress signals
    started = QtCore.Signal()
    pre_install_started = QtCore.Signal()
    pre_install_completed = QtCore.Signal()
    post_install_started = QtCore.Signal()
    post_install_completed = QtCore.Signal()
    cleanup_started = QtCore.Signal()
    cleanup_completed = QtCore.Signal()
    file_created = QtCore.Signal(str)
    done = QtCore.Signal(int)
    made_progress = QtCore.Signal(int, int)
    # Status signals
    directory_changed = QtCore.Signal(str)
    repo_changed = QtCore.Signal(Repository.Repository)

    # Exception
    class FailedException(Exception):
        """Raise to progress install dialog to result page with error description"""
        pass

    def __init__(self, install_dir: str = "", repo_name: str = "", api_token=""):
        super().__init__(parent=None)

        # Init data
        self.init_repo(api_token, repo_name)
        self.install_dir = install_dir
        if not self.install_dir.is_dir():
            self.install_dir = self.default_dir
        self.temp_path = pathlib.Path(os.getenv("TEMP"), pathlib.Path.cwd()) / f"temp_{self.repo.name}.zip"

    @property
    def default_dir(self):
        if platform.system() == "Windows":
            return os.getenv("programfiles", "")
        else:
            return ""

    @property
    def install_dir(self) -> pathlib.Path:
        return self._install_dir

    @install_dir.setter
    def install_dir(self, path):
        self._install_dir = pathlib.Path(path)
        self.directory_changed.emit(self._install_dir)

    @property
    def repo(self) -> Repository.Repository:
        return self._repo

    @repo.setter
    def repo(self, repo: Repository):
        self._repo = repo
        self.repo_changed.emit(repo)

    def init_repo(self, api_token, repo_name):
        git = Github(api_token)
        self.repo = git.get_repo(full_name_or_id=repo_name)

    def set_install_dir(self, path: str):
        self.install_dir = path

    def create_file(self, file_path: pathlib.Path, data="", encoding: str = None) -> None:
        Logger.info(f"Creating file {file_path.as_posix()}")
        try:
            file_path.write_text(data, encoding=encoding)
            self.file_created.emit(file_path.as_posix())
            Logger.info(f"Created file: {file_path.as_posix()}")
        except Exception:
            Logger.exception(f"Failed to create file {file_path.as_posix()}")
            raise Installer.FailedException()

    def download_latest_release(self):
        Logger.info(f"Downloading latest release for {self.repo.name}")
        try:
            release = self.repo.get_latest_release()
            self.create_file(self.temp_path)
            self.download_file(release.zipball_url, self.temp_path)
        except GithubException:
            Logger.exception(f"Failed to get latest release for {self.repo.full_name}")
            raise Installer.FailedException()

    @classmethod
    def download_file(cls, url: str, file_name: pathlib.Path):
        Logger.info(f"Downloading file {url}")
        try:
            with requests.get(url) as req_file:
                with open(file_name.as_posix(), "wb") as local_file:
                    for chunk in req_file.iter_content(chunk_size=8192):
                        if chunk:
                            local_file.write(chunk)
            Logger.info(f"Dowloaded file to: {file_name.as_posix()}")
            return file_name
        except Exception:
            Logger.exception(f"Failed to download from: {url}")
            raise Installer.FailedException

    def extract_file(self, source: pathlib.Path, destination_dir: pathlib.Path):
        Logger.info(f"Extracting temp file {source.as_posix()}")
        try:
            zip_file = ZipFile(source)
            root_folder = zip_file.namelist()[0]
            zip_file.extractall(destination_dir)
            extracted_path = destination_dir / root_folder
            final_path = destination_dir / self.repo.name
            if extracted_path.is_dir():
                answer = QtWidgets.QMessageBox.question(None,
                                                        "Existing directory",
                                                        f"Directory {final_path} already exists. Replace it?")
                if answer == QtWidgets.QMessageBox.Yes:
                    shutil.rmtree(final_path)
                    extracted_path.rename(final_path)
            else:
                extracted_path.rename(final_path)
            Logger.info(f"Extracted zip to {extracted_path.as_posix()}")
            return extracted_path
        except Exception:
            Logger.exception("Failed to extract file")
            raise Installer.FailedException

    def install(self) -> None:
        Logger.info("Installing...")
        self.download_latest_release()
        self.made_progress.emit(2, 6)
        self.extract_file(self.temp_path, self.install_dir)
        self.made_progress.emit(3, 6)

    def preInstall(self) -> None:
        """Override"""
        pass

    def postInstall(self) -> None:
        """Override"""
        pass

    def run(self) -> None:
        """Run installation"""
        Logger.info("Initializing ...")
        self.started.emit()
        self.made_progress.emit(0, 6)

        # Pre
        Logger.info("Running pre install tasks...")
        self.pre_install_started.emit()
        self.preInstall()
        self.pre_install_completed.emit()
        self.made_progress.emit(1, 6)
        Logger.info("Pre install tasks completed.")

        # Install
        Logger.info("Installing...")
        self.install()
        self.made_progress.emit(4, 6)
        Logger.info("Install tasks complete.")

        # Post
        Logger.info("Running post install tasks...")
        self.post_install_started.emit()
        self.postInstall()
        self.post_install_completed.emit()
        self.made_progress.emit(5, 6)
        Logger.info("Post install task completed.")

        # Cleanup
        Logger.info("Cleaning up...")
        self.cleanup_started.emit()
        os.remove(self.temp_path)
        self.cleanup_completed.emit()
        self.made_progress.emit(6, 6)
        self.done.emit(0)
        Logger.info("Done.")



import time
from PySide2 import QtWidgets
import simple_installer.common_widgets as common_widgets


class WelcomePage(QtWidgets.QWidget):
    def __init__(self, software_name: str = "Program", parent=None):
        super().__init__(parent)

        self.title_text = f"Welcome to the {software_name} Setup Wizard"
        self.welcome_text = f"""The Setup Wizard will install {software_name} on your computer.\nClick Next to continue or Cancel to exit Setup wizard"""

        self.create_widgets()
        self.create_layouts()
        self.create_connections()

    def create_widgets(self):
        self.title_label = QtWidgets.QLabel(self.title_text)
        self.title_label.setStyleSheet("font-weight: bold;")
        self.title_label.setWordWrap(1)
        self.message_label = QtWidgets.QLabel(self.welcome_text)
        self.message_label.setWordWrap(1)

    def create_layouts(self):
        self.main_layout = QtWidgets.QVBoxLayout(self)
        self.main_layout.addWidget(self.title_label)
        self.main_layout.addWidget(self.message_label)
        self.main_layout.addStretch()
        self.setLayout(self.main_layout)

    def create_connections(self):
        pass


class DirectoryPage(QtWidgets.QWidget):
    def __init__(self, default_dir: str, parent=None):
        super().__init__(parent)
        self.default_dir = default_dir
        self.create_widgets()
        self.create_layouts()
        self.create_connections()

    def create_widgets(self):
        self.dir_widget = common_widgets.DirectoryWidget(default_dir=self.default_dir)

    def create_layouts(self):
        self.main_layout = QtWidgets.QVBoxLayout(self)
        self.main_layout.addWidget(self.dir_widget)
        self.setLayout(self.main_layout)

    def create_connections(self):
        pass


class InstallPage(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.create_widgets()
        self.create_layouts()
        self.create_connections()

    def create_widgets(self):
        self.progress_bar = QtWidgets.QProgressBar()
        self.log_output = QtWidgets.QTextEdit()
        self.log_output.setReadOnly(True)

    def create_layouts(self):
        self.main_layout = QtWidgets.QVBoxLayout(self)
        self.main_layout.addWidget(self.progress_bar)
        self.main_layout.addWidget(self.log_output)
        self.setLayout(self.main_layout)

    def create_connections(self):
        pass

    def update_progress_bar(self, value: int, operations: int = 0):
        if not operations:
            self.progress_bar.setValue(value)
            return
        percentage = value / operations * 100
        self.progress_bar.setValue(percentage)
        time.sleep(0.3)



import sys
import inspect
import logging
import logging.handlers
from PySide2 import QtCore


class Logger:

    LOGGER_NAME = "installer"
    LEVEL_DEFAULT = logging.DEBUG
    PROPAGATE_DEFAULT = False
    FORMAT_DEFAULT = "[%(levelname)s] %(message)s"
    _logger_obj = None  # type: logging.Logger
    _signal_handler = None  # type: QSignalHandler

    @classmethod
    def logger_obj(cls):
        """Returns logger object

        :return: Logger object
        :rtype: logging.Logger
        """
        if not cls._logger_obj:
            if cls.logger_exists():
                cls._logger_obj = logging.getLogger(cls.LOGGER_NAME)
            else:
                cls._logger_obj = logging.getLogger(cls.LOGGER_NAME)
                cls._logger_obj.setLevel(cls.LEVEL_DEFAULT)
                cls.set_propagate(cls.PROPAGATE_DEFAULT)
                # Formatters
                fmt = logging.Formatter(cls.FORMAT_DEFAULT.format(cls.LOGGER_NAME), datefmt="%d-%m-%Y %H:%M:%S")
                qt_fmt = logging.Formatter("[%(levelname)s] %(message)s")
                # Handlers
                stream_handler = logging.StreamHandler(sys.stdout)
                stream_handler.setFormatter(fmt)
                cls._signal_handler = QSignalHandler()
                cls._signal_handler.setFormatter(qt_fmt)
                cls._logger_obj.addHandler(stream_handler)
                cls._logger_obj.addHandler(cls._signal_handler)

        return cls._logger_obj

    @classmethod
    def logger_exists(cls):
        return cls.LOGGER_NAME in logging.Logger.manager.loggerDict.keys()

    @classmethod
    def set_level(cls, level):
        lg = cls.logger_obj()
        lg.setLevel(level)

    @classmethod
    def get_level(cls, name=False):
        if name:
            return logging.getLevelName(cls.logger_obj().level)
        return cls.logger_obj().level

    @classmethod
    def set_propagate(cls, propagate):
        lg = cls.logger_obj()
        lg.propagate = propagate

    @classmethod
    def signal_handler(cls):
        cls.logger_obj()
        return cls._signal_handler

    @classmethod
    def call_info(cls, message):
        caller = inspect.getframeinfo(inspect.stack()[1][0])
        message = "file: {0} function {1}() lineno:{2}-{3}".format(caller.filename, caller.function, caller.lineno, message)

    @classmethod
    def debug(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.debug(msg, *args, **kwargs)

    @classmethod
    def info(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.info(msg, *args, **kwargs)

    @classmethod
    def warning(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.warning(msg, *args, **kwargs)

    @classmethod
    def error(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.error(msg, *args, **kwargs)

    @classmethod
    def critical(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.critical(msg, *args, **kwargs)

    @classmethod
    def log(cls, level, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.log(level, msg, *args, **kwargs)

    @classmethod
    def exception(cls, msg, *args, **kwargs):
        lg = cls.logger_obj()
        lg.exception(msg, *args, **kwargs)

    @classmethod
    def write_to_rotating_file(cls, path, level=logging.WARNING, mode="w", max_bytes=1024):
        lg = cls.logger_obj()
        if any([isinstance(handler, logging.handlers.RotatingFileHandler) for handler in lg.handlers]):
            lg.warning("Rotating file hander already exists")
            return

        rfile_hander = logging.handlers.RotatingFileHandler(path, mode=mode, maxBytes=max_bytes, backupCount=0, delay=0)
        rfile_hander.setLevel(level)
        fmt = logging.Formatter("[%(asctime)s][%(levelname)s] %(message)s")
        rfile_hander.setFormatter(fmt)
        lg.addHandler(rfile_hander)
        cls.info("Logging to file {}".format(path))


class QSignaler(QtCore.QObject):
    message_logged = QtCore.Signal(str)


class QSignalHandler(logging.Handler):
    def __init__(self, *args, **kwargs):
        super(QSignalHandler, self).__init__(*args, **kwargs)
        self.emitter = QSignaler()

    def emit(self, record):
        msg = self.format(record)
        self.emitter.message_logged.emit(msg)
        QtCore.QCoreApplication.processEvents()



from PySide2 import QtGui
from PySide2 import QtCore
from PySide2 import QtWidgets
from simple_installer import Logger
import simple_installer.installFn as installFn
import simple_installer.pages as pages


class InstallerWindow(QtWidgets.QMainWindow):
    def __init__(self, installer: installFn.Installer,
                 title: str = "Installer",
                 min_size: tuple = (400, 200),
                 initial_size=(400, 200)):
        super().__init__(parent=None)
        self.installer = installer
        self.create_actions()
        self.create_widgets()
        self.create_layouts()
        self.create_connections()

        self.setWindowTitle(title)
        self.setMinimumSize(*min_size)
        self.resize(*initial_size)
        self.update_buttons()

        # Apply dark pallete
        app_instance = QtCore.QCoreApplication.instance()
        app_instance.setStyle(QtWidgets.QStyleFactory.create("fusion"))
        app_instance.setPalette(dark_pallete())

    def create_actions(self):
        pass

    def create_widgets(self):
        # Central widget
        self.main_widget = QtWidgets.QWidget(self)
        self.setCentralWidget(self.main_widget)

        # Pages
        self.stack = QtWidgets.QStackedWidget()
        self.welcome_page = pages.WelcomePage(self.installer.repo.name)
        self.dir_page = pages.DirectoryPage(self.installer.install_dir.as_posix())
        self.install_page = pages.InstallPage()
        self.stack.addWidget(self.welcome_page)
        self.stack.addWidget(self.dir_page)
        self.stack.addWidget(self.install_page)

        # Buttons
        self.back_button = QtWidgets.QPushButton("< &Back")
        self.next_button = QtWidgets.QPushButton("&Next >")
        self.cancel_button = QtWidgets.QPushButton("Cancel")

    def create_layouts(self):
        self.buttons_layout = QtWidgets.QHBoxLayout()
        self.buttons_layout.addStretch()
        self.buttons_layout.addWidget(self.back_button)
        self.buttons_layout.addWidget(self.next_button)
        self.buttons_layout.addSpacing(20)
        self.buttons_layout.addWidget(self.cancel_button)

        self.main_layout = QtWidgets.QVBoxLayout()
        self.main_layout.addWidget(self.stack)
        self.main_layout.addLayout(self.buttons_layout)
        self.main_widget.setLayout(self.main_layout)

    def create_connections(self):
        self.next_button.clicked.connect(self.next_page)
        self.back_button.clicked.connect(self.previous_page)
        self.cancel_button.clicked.connect(self.close)
        self.stack.currentChanged.connect(self.update_buttons)
        self.dir_page.dir_widget.line_edit.textChanged.connect(self.installer.set_install_dir)
        Logger.signal_handler().emitter.message_logged.connect(self.install_page.log_output.append)
        self.installer.made_progress.connect(self.install_page.update_progress_bar)
        self.installer.done.connect(lambda: self.cancel_button.setText("Close"))

    @QtCore.Slot()
    def next_page(self):
        self.stack.setCurrentIndex(self.stack.currentIndex() + 1)
        if self.stack.currentIndex() == self.stack.indexOf(self.install_page):
            self.installer.run()

    @QtCore.Slot()
    def previous_page(self):
        current_index = self.stack.currentIndex()
        if current_index > 0:
            self.stack.setCurrentIndex(current_index - 1)

    @QtCore.Slot()
    def update_buttons(self):
        current_index = self.stack.currentIndex()
        self.back_button.setDisabled(current_index == 0)
        self.back_button.setVisible(current_index != self.stack.indexOf(self.install_page))
        self.next_button.setVisible(current_index != self.stack.indexOf(self.install_page))


def dark_pallete():
    palette = QtGui.QPalette()
    palette.setColor(QtGui.QPalette.Window, QtGui.QColor(45, 45, 45))
    palette.setColor(QtGui.QPalette.WindowText, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.Base, QtGui.QColor(25, 25, 25))
    palette.setColor(QtGui.QPalette.AlternateBase, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.ToolTipBase, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.ToolTipBase, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.Text, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.Button, QtGui.QColor(45, 45, 48))
    palette.setColor(QtGui.QPalette.ButtonText, QtGui.QColor(208, 208, 208))
    palette.setColor(QtGui.QPalette.BrightText, QtCore.Qt.red)
    palette.setColor(QtGui.QPalette.Link, QtGui.QColor(42, 130, 218))
    palette.setColor(QtGui.QPalette.Highlight, QtGui.QColor(42, 130, 218))
    palette.setColor(QtGui.QPalette.Highlight, QtCore.Qt.gray)
    return palette



import setuptools

with open("README.md", "r", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

requirements = []
with open('requirements.txt', 'r') as fh:
    for line in fh:
        requirements.append(line.strip())


setuptools.setup(
    name="simple_installer",
    version="2.0.1",
    author="Dmitrii Shevchenko",
    author_email="dmitrii.shevchenko96@gmail.com",
    description="Extendable installer for latest Github repository release.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/S0nic014/simple_installer",
    install_requires=requirements,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6"
)
